/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/**
 * NextGenUIMenuBar extends the menu bar from One UI to add support for creating
 * the menus dynamically from a collection. A menu bar is composed of multiple
 * popup menu bar items. An example of a popup menu bar item is the "Global"
 * menu. Each popup menu bar item then contains a menu dialog. The menu dialog
 * allows the popup menu bar item to contain not only contain a menu, but links
 * and search boxes as well. The menu inside a menu dialog can contain menu
 * items, menu headings, and a link area. The order that these items appear in
 * is determined by a weight attribute assigned to each item in the collection
 */

define("ibm/usmi/menus/NextGenUIMenuBar", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dojo/query",
	"idx/oneui/MenuBar",
	"dijit/PopupMenuBarItem",
	"dijit/MenuItem",
	"idx/oneui/MenuDialog",
	"idx/oneui/Menu",
	"dijit/form/TextBox",
	"dijit/form/Button",
	"ibm/usmi/util/DataStore",
	"ibm/usmi/core/utils/_UsmiBrowserPersistence",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"ibm/usmi/menus/NextGenUIMenuUtils",
	"ibm/usmi/core/utils/Logger",
	"dojo/topic",
	"dijit/registry",
	"dojo/_base/array",
	"dojo/dom-class"
], function (dojo, declare, WidgetBase, query, MenuBar, PopupMenuBarItem, MenuItem, MenuDialog, Menu, TextBox, Button,
		DataStore, _UsmiBrowserPersistence, MRI, NextGenUIMenuUtils, Logger, topic, registry, array, domClass) {
	return declare([
		WidgetBase,
		MenuBar,
		_UsmiBrowserPersistence
	], {

		_menuBar : null,

		_dataStore : null,

		_PERSISTENCE_MENUBAR_ID : "ibm.usmi.menus.NextGenUIMenuBar.contents",

		_nextGenUIMenuUtils : new NextGenUIMenuUtils({}),

		_nextGenUIMenuBarLogger : new Logger("ibm.usmi.menus", "NextGenUIMenuBar"),

		constructor : function (args) {
			dojo.safeMixin(this, args);

		},

		postCreate : function () {
			this.inherited(arguments);
			this._menuBar = this.getData(this._PERSISTENCE_MENUBAR_ID, true);
			if (this._nextGenUIMenuBarLogger.isInfo()) {
				this._nextGenUIMenuBarLogger.info("Checked session storage for menu bar, value is: " + this._menuBar,
						"postCreate");
			}
			if (this._menuBar) {
				this._createMenuBar();
			} else {
				if (this._nextGenUIMenuBarLogger.isInfo()) {
					this._nextGenUIMenuBarLogger.info(
							"Menu bar hasn't been stored yet, need to pull contents from server.", "postCreate");
				}
				this._dataStore = new DataStore({
					datasetId : 'HeaderMegaMenuCollection',
					homeParms : null,
					getItems : true,
					retrieveItemProperties : true,
					getTasks : false,
					initReadyCallback : dojo.hitch(this, this.initializeDataStores)
				});
			}
		},

		initializeDataStores : function () {
			var oneUIHeader=null;
			if (this._nextGenUIMenuBarLogger.isInfo()) {
				this._nextGenUIMenuBarLogger.info(
						"Menu bar data retrieved from server. Converting CURI data into the menu object",
						"initializeDataStores");
			}

			this._convertMenuCollectionToMenuObject();
			if (this._nextGenUIMenuBarLogger.isInfo()) {
				this._nextGenUIMenuBarLogger.info("Saving menu data to the session", "initializeDataStores");
			}
			this.setData(this._PERSISTENCE_MENUBAR_ID, this._menuBar, true);
			this._createMenuBar();
			oneUIHeader=registry.byId("oneUIHeader");
			if(oneUIHeader && oneUIHeader.secondaryNavigationInHeadShowed){
				//if OneUIHeader is presented and it has attempted to show the secondaryNavigation, 
				//we need to show the secondaryNavigation again to make sure all children of secondaryNavigation 
				//would be showed properly in OneUIHeader
				oneUIHeader._showSecondaryNavigationInHead();
				if (this._nextGenUIMenuBarLogger.isInfo()) {
					this._nextGenUIMenuBarLogger.info(
							"Rendering secondaryNavigation with OneUIHeader stuff",
							"initializeDataStores");
				}
			}
		},

		destroy : function () {
			// clean up the datastore
			if (this._dataStore) {
				this._dataStore.getDataset().removeEventListener(this);
				this._dataStore.destroy();
				this._dataStore = null;
			}
			
			this.inherited(arguments);
		},
        
		/**
		 * gets a menu from the menu bar object using its menu ID, Returns null
		 * if that menu doesn't exist in the menu bar
		 */
		_getMenuFromMenuBar : function (
		/*String*/menuId) {
			var i;
			for (i = 0; i < this._menuBar.menus.length; i++) {
				if (this._menuBar.menus[i].id === menuId) {
					return this._menuBar.menus[i];
				}
			}
			return null;

		},

		/**
		 * Finds and returns the link menu item from a given menu. Returns null
		 * if the menu does not contain a link menu item
		 */
		_getMenuLinksFromMenu : function (
		/*Array*/menu) {
			var i;
			for (i = 0; i < menu.items.length; i++) {
				if (menu.items[i].type === 'link') {
					return menu.items[i];
				}
			}
			return null;
		},

		/**
		 * converts a collection into a menu bar object for easy storage in
		 * session persistence
		 */
		_convertMenuCollectionToMenuObject : function () {
			if (this._nextGenUIMenuBarLogger.isFinest()) {
				this._nextGenUIMenuBarLogger.entering("", "_convertMenuCollectionToMenuObject");
			}
			var itemType, menuId, title, menu, items, sortedItems, links, i;

			items = this._dataStore.getItems();
			//sortedItems = new Array(items.length);
			sortedItems = [];
			for (i = 0; i < items.length; i++) {
				sortedItems[items[i].getProperty('ORDER_COLUMN').getValue()] = items[i];
			}

			this._menuBar = {
				"menus" : []
			};

			/*
				First create the menus themselves reading from the collection. 
				Once the menus are created, then read through the
				collection to populate them. 
			*/
			dojo.forEach(sortedItems, dojo.hitch(this, function (entry, i) {
				itemType = entry.getProperty('TYPE_COLUMN').getValue();
				menuId = entry.getProperty('MENU_ID_COLUMN').getValue();
				title = entry.getProperty('TITLE_COLUMN').getValue();

				if (itemType === 'menuBar') {
					// the help menu is currently it's own menu so don't include it here
					if (menuId !== 'HelpMenu') {

						this._menuBar.menus.push({
							"type" : itemType,
							"title" : title,
							"id" : menuId,
							"items" : []
						});
					}

				}
			}));

			/* populate the menus*/
			dojo.forEach(sortedItems, dojo.hitch(this, function (entry, i) {
				itemType = entry.getProperty('TYPE_COLUMN').getValue();
				menuId = entry.getProperty('MENU_ID_COLUMN').getValue();
				if (menuId !== 'HelpMenu') {
					menu = this._getMenuFromMenuBar(menuId);
					if (itemType === 'menuItem') {
						var oneitem = {
							"type" : itemType,
							"column" : entry.getProperty('COLUMN_COLUMN').getValue(),
							"title" : entry.getProperty('TITLE_COLUMN').getValue(),
							"task" : entry.getProperty('TASK_COLUMN').getValue(),
							"id" : menuId
						};
						menu.items.push(oneitem);
						this._increaseCount(menu.items, oneitem.column);
					} else if (itemType === 'menuHeading') {
						menu.items.push({
							"type" : itemType,
							"column" : entry.getProperty('COLUMN_COLUMN').getValue(),
							"title" : entry.getProperty('TITLE_COLUMN').getValue(),
							"id" : menuId,
							"count" : 0
						});
					} else if (itemType === 'menuHeadingImage') {
						menu.items.push({
							"type" : itemType,
							"column" : entry.getProperty('COLUMN_COLUMN').getValue(),
							"image" : entry.getProperty('IMAGE_COLUMN').getValue(),
							"id" : menuId
						});
					} else if (itemType === 'link') {
						links = this._getMenuLinksFromMenu(menu);
						if (!links) {
							links = {
								"type" : itemType,
								"id" : menuId,
								"links" : []
							};
							menu.items.push(links);
						}
						links.links.push({
							"id" : menuId,
							"title" : entry.getProperty('TITLE_COLUMN').getValue(),
							"task" : entry.getProperty('TASK_COLUMN').getValue()
						});

					}
				}
			}));
			
			// Go through menus to delete the empty columns
			dojo.forEach(this._menuBar.menus, dojo.hitch(this, function (menulist, i) {
				var copyMenus = new Array();
				var rowDeleted = -1;	// Which column is current empty
				var rowDeletedNum = 0;	// How many empty columns
				// Go inside each menuid
				dojo.forEach(menulist.items, dojo.hitch(this, function (entry, i) {
					var skipped = false;
					if (entry.type === "menuHeading") {
						if (entry.count === 0) {
							// Mark the empty column
							rowDeleted = parseInt(entry.column);
							rowDeletedNum += 1;
							skipped = true;
						}
					} 
					if (entry.column) {
						if (rowDeletedNum > 0) {
							if (parseInt(entry.column)>rowDeleted) {
								// Move the following columns
								entry.column = (parseInt(entry.column)-rowDeletedNum)+"";
							}
						}
					}
					if (!skipped) {					
						copyMenus.push(entry);					
					}
				}));
				// Copy back the processed items
				menulist.items = copyMenus;
			}));			
			
			if (this._nextGenUIMenuBarLogger.isFinest()) {
				this._nextGenUIMenuBarLogger.exiting("", "_convertMenuCollectionToMenuObject");
			}
		},
		
		/**
		 * Count items under one menu heading
		 */
		_increaseCount : function (items, column) {
			dojo.forEach(items, dojo.hitch(this, function (entry, i) {				
				if (entry.column === column && entry.type === "menuHeading") {
					entry.count += 1;
				}				
			}));
		},		

		/**
		 * creates the menu headings from a menu bar object that's created from
		 * a collection, creates a new popup menu bar item, menu dialog, and
		 * submenu for each one.
		 */
		_createMenuBar : function () {
			if (this._nextGenUIMenuBarLogger.isFinest()) {
				this._nextGenUIMenuBarLogger.entering("Creating a menu bar with " + this._menuBar.menus.length
						+ " menus", "_createMenuBar");
			}
			// the popup menu being parsed
			var popup,
			// the menu dialog that will contain all the popup menus
			menuDialog, menuTitle, i;

			// only parse the menu headings if the menu bar data has been set
			if (this._menuBar) {
				this.menuHeadersParsed = true;

				//now that we've stored the menu headers in the menu bar object, create the actual menus
				//dojo.forEach(this._menuBar.menus, dojo.hitch(this, function (entry, i) {
				for (i = 0; i < this._menuBar.menus.length; i++) {
					menuTitle = this._menuBar.menus[i].title;
					if (this._nextGenUIMenuBarLogger.isFinest()) {
						this._nextGenUIMenuBarLogger.finest("Creating menu: " + menuTitle, "_createMenuBar");
					}
					popup = new PopupMenuBarItem({});
					menuDialog = new MenuDialog();
					popup.set("label", MRI[menuTitle]);
					menuDialog = new MenuDialog({});
					popup.set("popup", menuDialog);
					//explicitly set an ID so that we can find the widget later to assign
					//it its submenu
					popup.set("id", this._menuBar.menus[i].id);
					this.addChild(popup);
				}
			}

			if (this._nextGenUIMenuBarLogger.isInfo()) {
				this._nextGenUIMenuBarLogger.info("Publishing topic that the menu bar is loaded", "_createMenuBar");
			}

			topic.publish("nextgen-ui-menubar-loaded", null);

			if (this._nextGenUIMenuBarLogger.isFinest()) {
				this._nextGenUIMenuBarLogger.exiting("", "_createMenuBar");
			}
		},

		/**
		 * creates the contents of a popupmenu using the menu bar object
		 * constructed using the collection. In order the parse the menu
		 * contents, the menu headers must be parsed first. If the
		 * parseMenuHeadings() function hasn't been called, then it gets called
		 * in this function
		 */
		populateMenuBarMenus : function () {
			if (this._nextGenUIMenuBarLogger.isFinest()) {
				this._nextGenUIMenuBarLogger.entering("", "_populateMenuBarMenus");
			}
			//the menu being populated
			var menu,
			//the widget ID of the menu being populated
			menuId, i, j,
			//the menu items in a menu
			items,
			//stores the contents of a menu while parsing through a menu, 
			//used to set the contents of the menu dialog for a menu
			nodeList,
			//reference to the popup mwnu widget in the menu bar
			popup,
			//true if the menu, where the menu items are contained, has been included in the nodelist
			menuIncluded = false;

			if (this._menuBar) {
				for (i = 0; i < this._menuBar.menus.length; i++) {
					items = this._menuBar.menus[i].items;
					menuId = this._menuBar.menus[i].id;
					//each popup menu contains a submenu, so create it right away
					menu = new Menu({
						"class" : "menu-workitem"
					});
					nodeList = new query.NodeList();
					var headings = [];
					for (j = 0; j < items.length; j++) {
						if (items[j].type === 'menuItem') {
							headings[items[j].column] = true;
						}
					}
					for (j = 0; j < items.length; j++) {
						//check to see if the item type corresponds to an item contained in the menu. If it does
						//push the menu onto the nodelist if it hasn't been pushed on already
						if (items[j].type === 'menuItem' || items[j].type === 'menuHeading'
								|| items[j].type === 'menuHeadingImage') {
							if (!menuIncluded) {
								menuIncluded = true;
								nodeList.push(menu.domNode);
							}
						}
						if (items[j].type === 'menuItem') {
							menu.addChild((this._nextGenUIMenuUtils.createMenuItem(items[j].title, items[j].column,
									items[j].task, MRI)));
						} else if (items[j].type === 'menuHeading') {
							if (headings[items[j].column]) {
								menu.addChild(this._nextGenUIMenuUtils.createMenuHeading(items[j].title,
										items[j].column, MRI));
							}
						} else if (items[j].type === 'menuHeadingImage') {
							menu.addChild(this._nextGenUIMenuUtils.createMenuHeadingImage(items[j].image,
									items[j].column));
						} else if (items[j].type === 'link') {
							nodeList.push(this._nextGenUIMenuUtils.createLinks(items[j].links, MRI));
						}
					}

					popup = this._nextGenUIMenuUtils.getWidgetFromRegistry(menuId);
					if (popup) {
						popup.popup.set("content", nodeList);
					} else {
						if (this._nextGenUIMenuBarLogger.isWarn()) {
							this._nextGenUIMenuBarLogger.warn(
									"Popup widget doesn't exist in the registry. Unable to add menu to popup a menu.",
									"_populateMenuBarMenus");
						}
					}

					menuIncluded = false;
				}
			} else {
				if (this._nextGenUIMenuBarLogger.isWarn()) {
					this._nextGenUIMenuBarLogger.warn("The menu bar object hasn't been created yet.",
							"_populateMenuBarMenus");
				}
			}
			if (this._nextGenUIMenuBarLogger.isFinest()) {
				this._nextGenUIMenuBarLogger.exiting("", "_populateMenuBarMenus");
			}
		},
		
		
		onItemHover: function(/*MenuItem*/ item){
			var childMenuItems;
			this.inherited(arguments);
			childMenuItems=this.getChildren();
			//hack the defect XB126110
			array.forEach(childMenuItems, function(menuBarItem){
				if(menuBarItem === item){
					return;
				}
				if(domClass.contains(menuBarItem.domNode, "dijitMenuItemSelected")){
					domClass.remove(menuBarItem.domNode, "dijitMenuItemSelected");
					//console.log("remove the selected css class");
				}
				
			});
			
		}

	});

});
